#R code for Steinthorsdottir et al, "Near-Future pCO2 during the hot Mid Miocene Climatic Optimum"
#pCO2 Monte Carlo error propagation
#To run R code, save Table S4 as a .csv file, with the filename "Clarkia.csv"

#Read in data
co2 <- read.csv("Clarkia.csv", header = T, row.names = 1)

##Plant C3 proxy: Schubert and Jahren (2012, 2015) model with Cui and Schubert (2016) Monte Carlo approach
#CO2 estimated relative to Holocene baseline
#Code adapted from Lomax et al (2019)
nrep <- 10000
sj.co2 <- matrix(0, nrow = nrow(co2), ncol = nrep)
rownames(sj.co2) <- rownames(co2)

for (i in 1:nrow(sj.co2)) {
  A <- 28.26
  B <- rnorm(nrep, 0.22, 0.028)
  C <- (4.4*A)/((A-4.4)*B)
  d13C.org.t <- rnorm(nrep, co2$d13Cp[i], co2$ed13Cp[i])
  d13C.co2.t <- rnorm(nrep, -5.3, 0.2)
  D13C.t <- (d13C.co2.t - d13C.org.t)/(1 + d13C.org.t/1000)
  d13C.org.t0 <- rnorm(nrep, -25.1, 1.6)
  d13C.co2.t0 <- rnorm(nrep, -6.4, 0.1)
  D13C.t0 <- (d13C.co2.t0 - d13C.org.t0)/(1 + d13C.org.t0/1000)
  DD13C <- D13C.t - D13C.t0
  pCO2.t0 <- rnorm(nrep, 270, 7)
  
  sj.co2[i,] <- (DD13C*A^2 + DD13C*A*B*pCO2.t0 + 2*DD13C*A*B*C + DD13C*B^2*C*pCO2.t0 +
                   DD13C*B^2*C^2 + A^2*B*pCO2.t0)/
    (A^2*B - DD13C*A*B - DD13C*B^2*pCO2.t0 - DD13C*B^2*C)
}

#Number of unrealistic CO2 estimates (below 0 or above 10^6)
(sj.co2.anom <- data.frame(t.lo = sum(sj.co2 < 0),
                          t.hi = sum(sj.co2 > 1e+06)))
sj.co2[sj.co2 < 0 | sj.co2 > 1e+06] <- NA #Remove from estimates

#a. Percentiles for each sample
sj.percentiles <- t(apply(sj.co2, MARGIN = 1, FUN = quantile,
                               probs = c(0.025, 0.16, 0.5, 0.84, 0.975), na.rm = T))
write.csv(sj.percentiles, "SJ_percentiles.csv")

#b. Within taxon percentiles for each locality
sj.p37.percentiles <- rbind(quantile(as.vector(sj.co2[co2$Locality=="P37" & co2$Taxon=="Bev",]),
                                        c(0.025, 0.16, 0.5, 0.84, 0.975),
                                        na.rm = T),
                               quantile(as.vector(sj.co2[co2$Locality=="P37" & co2$Taxon=="Bef",]),
                                        c(0.025, 0.16, 0.5, 0.84, 0.975),
                                        na.rm = T),
                               quantile(as.vector(sj.co2[co2$Locality=="P37" & co2$Taxon=="Que",]),
                                        c(0.025, 0.16, 0.5, 0.84, 0.975),
                                        na.rm = T))
rownames(sj.p37.percentiles) <- c("Bev", "Bef", "Que")

sj.p33.percentiles <- rbind(quantile(as.vector(sj.co2[co2$Locality=="P33" & co2$Taxon=="Bev",]),
                                        c(0.025, 0.16, 0.5, 0.84, 0.975),
                                        na.rm = T),
                               quantile(as.vector(sj.co2[co2$Locality=="P33" & co2$Taxon=="Bef",]),
                                        c(0.025, 0.16, 0.5, 0.84, 0.975),
                                        na.rm = T),
                               quantile(as.vector(sj.co2[co2$Locality=="P33" & co2$Taxon=="Que",]),
                                        c(0.025, 0.16, 0.5, 0.84, 0.975),
                                        na.rm = T),
                               quantile(as.vector(sj.co2[co2$Locality=="P33" & co2$Taxon=="Laur",]),
                                        c(0.025, 0.16, 0.5, 0.84, 0.975),
                                        na.rm = T),
                               quantile(as.vector(sj.co2[co2$Locality=="P33" & co2$Taxon=="Ost",]),
                                        c(0.025, 0.16, 0.5, 0.84, 0.975),
                                        na.rm = T))
rownames(sj.p33.percentiles) <- c("Bev", "Bef", "Que", "Laur", "Ost")

#c. Within locality percentiles
sj.locality.percentiles <- rbind(quantile(as.vector(sj.co2[co2$Locality=="P37",]),
                                              c(0.025, 0.16, 0.5, 0.84, 0.975),
                                              na.rm = T),
                                     quantile(as.vector(sj.co2[co2$Locality=="P33",]),
                                              c(0.025, 0.16, 0.5, 0.84, 0.975),
                                              na.rm = T))
rownames(sj.locality.percentiles) <- c("P37", "P33")


##Stomatal ratio and transfer functions
#Separate dataset into taxa
laur <- co2[co2$Taxon == "Laur",]
bet <- co2[co2$Taxon == "Bev" | co2$Taxon == "Bef" | co2$Taxon == "Ost",]
quer <- co2[co2$Taxon == "Que",]

#Stomatal ratio
nrep <- 10000
#Lauraceae
laur.sr.mc <- matrix(0, nrow = nrow(laur), ncol = nrep,
                     dimnames = list(rownames(laur)))

for (i in 1:nrow(laur)) {
  sr.fos <- rnorm(nrep, laur$SI[i], laur$eSI_sem[i])
  sr.lseb <- rnorm(nrep, 19, 0.81)
  sr.lfus <- rnorm(nrep, 18.3, 0.66)
  sr.lsto <- rnorm(nrep, 14.4, 0.68)
  sr.lnob <- rowMeans(cbind(rnorm(nrep, 16.37, 0.54),
                            rnorm(nrep, 17.1, 0.73),
                            rnorm(nrep, 19.49, 0.18),
                            rnorm(nrep, 20.32, 0.15)))
  
  laur.sr.mc[i,] <- rowMeans(cbind(sr.lseb/sr.fos*360,
                                sr.lfus/sr.fos*360,
                                sr.lsto/sr.fos*360,
                                sr.lnob/sr.fos*300))
}

#Betulaceae
bet.sr.mc <- matrix(0, nrow = nrow(bet), ncol = nrep,
                    dimnames = list(rownames(bet)))

for (i in 1:nrow(bet)) {
  sr.fos <- rnorm(nrep, bet$SI[i], bet$eSI_sem[i])
  sr.blum <- rnorm(nrep, 10.7, 0.19)
  sr.bpen <- rowMeans(cbind(rnorm(nrep, 6.85, 0.15),
                            rnorm(nrep, 7.1, 0.53),
                            rnorm(nrep, 7.95, 0.42),
                            rnorm(nrep, 8.25, 0.32)))
  
  bet.sr.mc[i,] <- rowMeans(cbind(sr.blum/sr.fos*390,
                                  sr.bpen/sr.fos*360))
}

#Quercus
quer.sr.mc <- matrix(0, nrow = nrow(quer), ncol = nrep,
                     dimnames = list(rownames(quer)))

for (i in 1:nrow(quer)) {
  sr.fos <- rnorm(nrep, quer$SI[i], quer$eSI_sem[i])
  sr.qrob.qpet <- rnorm(nrep, 18.5, 0.57)
  sr.qrob <- rnorm(nrep, 21, 0.5)
  sr.qnig1 <- rnorm(nrep, 22, 0.51)
  sr.qnig2 <- rnorm(nrep, 19.5, 0.86)
  
  quer.sr.mc[i,] <- rowMeans(cbind(sr.qrob.qpet/sr.fos*290,
                                   sr.qrob/sr.fos*345,
                                   sr.qnig1/sr.fos*320,
                                   sr.qnig2/sr.fos*360))
}

#Bring together and put in sample order
sr.co2 <- rbind(laur.sr.mc, bet.sr.mc, quer.sr.mc)
sr.co2 <- sr.co2[match(rownames(co2), rownames(sr.co2)),]

#Any unrealistic CO2 estimates?
(sr.co2.anom <- data.frame(t.lo = sum(sr.co2 < 0, na.rm = T),
                           t.hi = sum(sr.co2 > 1e+06, na.rm = T)))
sr.co2[sr.co2 < 0 | sr.co2 > 1e+06] <- NA

#a. Percentiles for each sample
sr.percentiles <- t(apply(sr.co2, MARGIN = 1, FUN = quantile,
                          probs = c(0.025, 0.16, 0.5, 0.84, 0.975), na.rm = T))
write.csv(sr.percentiles, "SR_percentiles.csv")

#b. Within taxon percentiles for each locality
sr.p37.percentiles <- rbind(quantile(as.vector(sr.co2[co2$Locality=="P37" & co2$Taxon=="Bev",]),
                                     c(0.025, 0.16, 0.5, 0.84, 0.975),
                                     na.rm = T),
                            quantile(as.vector(sr.co2[co2$Locality=="P37" & co2$Taxon=="Bef",]),
                                     c(0.025, 0.16, 0.5, 0.84, 0.975),
                                     na.rm = T),
                            quantile(as.vector(sr.co2[co2$Locality=="P37" & co2$Taxon=="Que",]),
                                     c(0.025, 0.16, 0.5, 0.84, 0.975),
                                     na.rm = T))
rownames(sr.p37.percentiles) <- c("Bev", "Bef", "Que")

sr.p33.percentiles <- rbind(quantile(as.vector(sr.co2[co2$Locality=="P33" & co2$Taxon=="Bev",]),
                                     c(0.025, 0.16, 0.5, 0.84, 0.975),
                                     na.rm = T),
                            quantile(as.vector(sr.co2[co2$Locality=="P33" & co2$Taxon=="Bef",]),
                                     c(0.025, 0.16, 0.5, 0.84, 0.975),
                                     na.rm = T),
                            quantile(as.vector(sr.co2[co2$Locality=="P33" & co2$Taxon=="Que",]),
                                     c(0.025, 0.16, 0.5, 0.84, 0.975),
                                     na.rm = T),
                            quantile(as.vector(sr.co2[co2$Locality=="P33" & co2$Taxon=="Laur",]),
                                     c(0.025, 0.16, 0.5, 0.84, 0.975),
                                     na.rm = T),
                            quantile(as.vector(sr.co2[co2$Locality=="P33" & co2$Taxon=="Ost",]),
                                     c(0.025, 0.16, 0.5, 0.84, 0.975),
                                     na.rm = T))
rownames(sr.p33.percentiles) <- c("Bev", "Bef", "Que", "Laur", "Ost")

#c. Within locality percentiles
sr.locality.percentiles <- rbind(quantile(as.vector(sr.co2[co2$Locality=="P37",]),
                                          c(0.025, 0.16, 0.5, 0.84, 0.975),
                                          na.rm = T),
                                 quantile(as.vector(sr.co2[co2$Locality=="P33",]),
                                          c(0.025, 0.16, 0.5, 0.84, 0.975),
                                          na.rm = T))
rownames(sr.locality.percentiles) <- c("P37", "P33")


#Transfer functions
nrep <- 10000
#Lauraceae
laur.tf.mc <- matrix(0, nrow = nrow(laur), ncol = nrep,
                     dimnames = list(rownames(laur)))

for (i in 1:nrow(laur)) {
  tf.fos <- rnorm(nrep, laur$SI[i], laur$eSI_sem[i])
  
  laur.tf.mc[i,] <- (10^(3.173 - (0.5499*log10(tf.fos))))+150
}

#Betulaceae
bet.tf.mc <- matrix(0, nrow = nrow(bet), ncol = nrep,
                    dimnames = list(rownames(bet)))

for (i in 1:nrow(bet)) {
  tf.fos <- rnorm(nrep, bet$SI[i], bet$eSI_sem[i])
  
  bet.tf.mc[i,] <- rowMeans(cbind(10^(2.8687-(0.3579*log10(tf.fos))),
                                  (tf.fos-20.58)/-0.026))
}

#Quercus
quer.tf.mc <- matrix(0, nrow = nrow(quer), ncol = nrep,
                     dimnames = list(rownames(quer)))

for (i in 1:nrow(quer)) {
  tf.fos <- rnorm(nrep, quer$SI[i], quer$eSI_sem[i])
  
  quer.tf.mc[i,] <- -6.25*(tf.fos-71.09)
}

#Bring together and put in sample order
tf.co2 <- rbind(laur.tf.mc, bet.tf.mc, quer.tf.mc)
tf.co2 <- tf.co2[match(rownames(co2), rownames(tf.co2)),]

#Any unrealistic CO2 estimates?
(tf.co2.anom <- data.frame(t.lo = sum(tf.co2 < 0, na.rm = T),
                           t.hi = sum(tf.co2 > 1e+06, na.rm = T)))
tf.co2[tf.co2 < 0 | tf.co2 > 1e+06] <- NA

#a. Percentiles for each sample
tf.percentiles <- t(apply(tf.co2, MARGIN = 1, FUN = quantile,
                          probs = c(0.025, 0.16, 0.5, 0.84, 0.975), na.rm = T))
write.csv(tf.percentiles, "TF_percentiles.csv")

#b. Within taxon percentiles for each locality
tf.p37.percentiles <- rbind(quantile(as.vector(tf.co2[co2$Locality=="P37" & co2$Taxon=="Bev",]),
                                     c(0.025, 0.16, 0.5, 0.84, 0.975),
                                     na.rm = T),
                            quantile(as.vector(tf.co2[co2$Locality=="P37" & co2$Taxon=="Bef",]),
                                     c(0.025, 0.16, 0.5, 0.84, 0.975),
                                     na.rm = T),
                            quantile(as.vector(tf.co2[co2$Locality=="P37" & co2$Taxon=="Que",]),
                                     c(0.025, 0.16, 0.5, 0.84, 0.975),
                                     na.rm = T))
rownames(tf.p37.percentiles) <- c("Bev", "Bef", "Que")

tf.p33.percentiles <- rbind(quantile(as.vector(tf.co2[co2$Locality=="P33" & co2$Taxon=="Bev",]),
                                     c(0.025, 0.16, 0.5, 0.84, 0.975),
                                     na.rm = T),
                            quantile(as.vector(tf.co2[co2$Locality=="P33" & co2$Taxon=="Bef",]),
                                     c(0.025, 0.16, 0.5, 0.84, 0.975),
                                     na.rm = T),
                            quantile(as.vector(tf.co2[co2$Locality=="P33" & co2$Taxon=="Que",]),
                                     c(0.025, 0.16, 0.5, 0.84, 0.975),
                                     na.rm = T),
                            quantile(as.vector(tf.co2[co2$Locality=="P33" & co2$Taxon=="Laur",]),
                                     c(0.025, 0.16, 0.5, 0.84, 0.975),
                                     na.rm = T),
                            quantile(as.vector(tf.co2[co2$Locality=="P33" & co2$Taxon=="Ost",]),
                                     c(0.025, 0.16, 0.5, 0.84, 0.975),
                                     na.rm = T))
rownames(tf.p33.percentiles) <- c("Bev", "Bef", "Que", "Laur", "Ost")

#c. Within locality percentiles
tf.locality.percentiles <- rbind(quantile(as.vector(tf.co2[co2$Locality=="P37",]),
                                          c(0.025, 0.16, 0.5, 0.84, 0.975),
                                          na.rm = T),
                                 quantile(as.vector(tf.co2[co2$Locality=="P33",]),
                                          c(0.025, 0.16, 0.5, 0.84, 0.975),
                                          na.rm = T))
rownames(tf.locality.percentiles) <- c("P37", "P33")


##Re-fitting transfer functions with error on model coefficients as well as stomatal index

#Kürschner et al (2008) data: log CO2 regressed onto log SI
#Standard error on intercept = 0.097, standard error on slope = 0.0077

#Re-run Monte Carlo on Lauraceae data using Kürschner et al transfer function with errors on model coefficients
nrep <- 10000
kur.tf.mc <- matrix(0, nrow = nrow(laur), ncol = nrep,
                    dimnames = list(rownames(laur)))

for (i in 1:nrow(laur)) {
  tf.fos <- rnorm(nrep, laur$SI[i], laur$eSI_sem[i])
  tf.int <- rnorm(nrep, 3.173, 0.097)
  tf.slope <- rnorm(nrep, 0.5499, 0.0077)
  
  kur.tf.mc[i,] <- (10^(tf.int - (tf.slope*log10(tf.fos))))+150
}

#Any unrealistic CO2 estimates?
(kur.tf.mc.anom <- data.frame(t.lo = sum(kur.tf.mc < 0, na.rm = T),
                              t.hi = sum(kur.tf.mc > 1e+06, na.rm = T)))
kur.tf.mc[kur.tf.mc < 0 | kur.tf.mc > 1e+06] <- NA

#Calculate percentiles of Monte Carlo distribution
kur.tf.percentiles <- quantile(as.vector(kur.tf.mc),
                               c(0.025, 0.16, 0.5, 0.84, 0.975),
                               na.rm = T)

#Sun et al (2012) data: SI regressed onto CO2
#Standard error on intercept = 1.427, standard error on slope = 0.0004

#Re-run Monte Carlo on Betulaceae data using Sun et al transfer function with errors on model coefficients
nrep <- 10000
sun.tf.mc <- matrix(0, nrow = nrow(bet), ncol = nrep,
                    dimnames = list(rownames(bet)))

for (i in 1:nrow(bet)) {
  tf.fos <- rnorm(nrep, bet$SI[i], bet$eSI_sem[i])
  tf.int <- rnorm(nrep, -20.58, 1.427)
  tf.slope <- rnorm(nrep, -0.026, 0.0004)
  
  sun.tf.mc[i,] <- (tf.fos+tf.int)/tf.slope
}

#Any unrealistic CO2 estimates?
(sun.tf.mc.anom <- data.frame(t.lo = sum(sun.tf.mc < 0, na.rm = T),
                              t.hi = sum(sun.tf.mc > 1e+06, na.rm = T)))
sun.tf.mc[sun.tf.mc < 0 | sun.tf.mc > 1e+06] <- NA

#Calculate percentiles of Monte Carlo distribution
sun.tf.locality.percentiles <- rbind(quantile(as.vector(sun.tf.mc[bet$Locality=="P37",]),
                                              c(0.025, 0.16, 0.5, 0.84, 0.975),
                                              na.rm = T),
                                     quantile(as.vector(sun.tf.mc[bet$Locality=="P33",]),
                                              c(0.025, 0.16, 0.5, 0.84, 0.975),
                                              na.rm = T))
rownames(sun.tf.locality.percentiles) <- c("P37", "P33")

#For direct comparison, re-run Monte Carlo on Betulaceae data using just Sun et al transfer function with only errors on stomatal index
nrep <- 10000
sun.tf.mc.2 <- matrix(0, nrow = nrow(bet), ncol = nrep,
                      dimnames = list(rownames(bet)))

for (i in 1:nrow(bet)) {
  tf.fos <- rnorm(nrep, bet$SI[i], bet$eSI_sem[i])
  
  sun.tf.mc.2[i,] <- (tf.fos-20.58)/-0.026
}

#Any unrealistic CO2 estimates?
(sun.tf.mc.2.anom <- data.frame(t.lo = sum(sun.tf.mc.2 < 0, na.rm = T),
                              t.hi = sum(sun.tf.mc.2 > 1e+06, na.rm = T)))
sun.tf.mc.2[sun.tf.mc.2 < 0 | sun.tf.mc.2 > 1e+06] <- NA

#Calculate percentiles of Monte Carlo distribution
sun.tf.locality.percentiles.2 <- rbind(quantile(as.vector(sun.tf.mc.2[bet$Locality=="P37",]),
                                              c(0.025, 0.16, 0.5, 0.84, 0.975),
                                              na.rm = T),
                                     quantile(as.vector(sun.tf.mc.2[bet$Locality=="P33",]),
                                              c(0.025, 0.16, 0.5, 0.84, 0.975),
                                              na.rm = T))
rownames(sun.tf.locality.percentiles.2) <- c("P37", "P33")
